<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Abstract.php 3536 2014-04-07 12:30:24Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Db_Table_Abstract
 */
require_once 'Zend/Db/Table/Abstract.php';

/**
 * Abstract class for table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_Layer_Abstract extends Giscuit_Db_Table_Abstract
{

    /**
     * Table primary key name
     * 
     * @var string
     */
    protected $_primary = 'gid';

    /**
     * Holds layer settings
     * 
     * @var array
     */
    protected $_settings;

    /**
     * Is data from this table being fetched from the database
     * 
     * @var bool
     */
    protected $_fetchData = true;

    /**
     * List of additional fields that are pulled from the database at map rendering
     * 
     * @var array
     */
    public $styleFields = array();
    
    /**
     * Tolerance in pixels. Used for info query
     * 
     * @var integer
     */
    public $tolerance = 5;

    /**
     * Zoom to specified scale on search. Useful for point layers those bounding box 
     * could not be computed 
     * 
     * @var integer
     */
    public $searchScale;

    /**
     * From this scale snap is being enabled
     * 
     * @var integer
     */
    public $snapScale;

    /**
     * From this scale edit is being enabled
     * 
     * @var integer
     */
    public $editScale;

    /**
     * Holds the join statements
     * 
     * @var array 
     */
    protected $_joinQueue = array();

    /**
     * Table default field
     * 
     * @var string
     */
    protected $_defaultField = '';

    /**
     * PostGIS measurement functions
     * 
     * @var array
     */
    public static $measurementFuncts = array(
        "ST_Area" ,
        "ST_Length" ,
        "ST_Perimeter"
    );
    
    /**
     * Layer
     * 
     * @var Giscuit_Map_Layer_Vector
     */
    public $layer;
    
    /**
     * Constructor
     *
     * @param array $options Available options
     * @return void
     */
    public function __construct ($options = array())
    {
        $config = Zend_Registry::get('config');

        if(empty($this->_schema)) {
            $this->_schema = $config->database->schema . '_layers';
            $this->_mainSchema = $config->database->schema;
        }
        $this->_layersSchema = $config->database->schema . '_layers';
        $this->_historySchema = $config->database->schema . '_history';
        
        parent::__construct($options);
    }

    /**
     * Initialize
     *
     * @return void
     */
    public function init ()
    {
        parent::init();
    }

    /**
     * Set settings
     *
     * @param array $settings Settings
     * @return void
     */
    public function setSettings ($settings)
    {
        $config = Zend_Registry::get('config');
        
        $this->_settings = $settings;
        
        //Default field
        if (! empty($this->_settings['default_field'])) {
            $this->_defaultField = $this->_settings['default_field'];
        }
        
        //Edit maxscale
        $this->editScale = intval($this->_settings['edit_maxscale']);
        
        //Snap maxscale
        $this->snapScale = intval($this->_settings['snap_maxscale']);

        //Disable layer if it's currently being edited
        if ($this->layer &&
                
            isset($settings['edit']) && 
            $settings['edit'] == 't' && 
            
            isset(Zend_Registry::get('settings')->editingActive) &&
            Zend_Registry::get('settings')->editingActive == true
            ) {

            $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
            $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();

            if (
                //Check user_id
                $this->_settings['permissions_user_id'] == $userId ||
                    
                //Check priv_editgeom
                strstr(',' . $this->_settings['permissions_editgeom_ids'], ',' . $userId . ',') ||
                    
                //Check resource
                ! empty($this->_settings['priv_editgeom']) && $isAllowed->direct($this->_settings['resource'], $this->_settings['priv_editgeom']) ||
                ! empty($this->_settings['priv_editattr']) && $isAllowed->direct($this->_settings['resource'], $this->_settings['priv_editattr'])
                ) {
                $tableName = 'Giscuit_Db_Table_Layer_' . $this->_settings['class'];
                
                if ($this->layer->parent->addEditLayer == false && 
                    get_class($this) == $tableName 
                    && $this->layer->parent->getScale() <= $this->editScale
                    ) {
                    
                    $this->_fetchData = false;
                }
            }
        
        }
    
    }

    /**
     * Get settings
     *
     * @return array
     */
    public function getSettings ()
    {
        return $this->_settings;
    }
    
    /**
     * Get the current select object of table
     *  
     * @param string $dump - What data should be requested from the layer (gid_geometry, gid_geometry_label, all, all_table, count)
     * @param array $options Available options:
     * <ul><li>geometryAsWkt bool - Return geometry in WKT format</li>
     * <li>geometryAsGeoJSON bool - Return geometry in GeoJSON format</li>
     * <li>withIntersection string - Table name and geometry field (table.the_geom) that will be intersected with the current geometry. Returns the product of the intersection as the geometry.</li>
     * <li>whereIntersection string - Fetch rows that intersect specified table name and geometry (table.the_geom)</li>
     * <li>whereStm string - Fetch rows that match the specified statement</li>
     * <li>fields array - An array [[name, visibility, expr]] of additional field to fetch</li>
     * <li>form Zend_Form - Edit attributes form object of this layer</li>
     * <li>limit integer - Number of rows to fecth</li></ul>
     * @return Zend_Db_Select
     */
    public function getSelect ($dump = 'gid_geometry_label', array $options = array())
    {
        if ($this->_fetchData == false) {
            return;
        }
        
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        //WKT
        if (isset($options['geometryAsWkt']) && $options['geometryAsWkt'] == true) {
            $theGeomField = new Zend_Db_Expr('ST_AsText(' . $this->getAdapter()->quoteIdentifier($this->_name . '.the_geom') . ')');

        //GeoJSON
        } else if (isset($options['geometryAsGeoJSON']) && $options['geometryAsGeoJSON'] == true) {
            $theGeomField = new Zend_Db_Expr('ST_AsGeoJSON(' . $this->getAdapter()->quoteIdentifier($this->_name . '.the_geom') . ')');
            
        //            
        } else {
            $theGeomField = $this->_name . '.the_geom';
        }
        
        $fields = array(
            $this->getPrimaryKey(array('name' => true)) => $this->getPrimaryKey() , 
            'the_geom' => $theGeomField
        );
        
        //
        $config = Zend_Registry::get('config');
        
        if ($config->map->units == 'dd') {
            $geodesicStmt = ', true';
        } else {
            $geodesicStmt = '';
        }
        
        //
        if ($dump == 'gid_geometry_label') {
            $fields = $this->addLabelFields($select, $fields);
            
        } elseif ($dump == 'gid_geometry') {

        } elseif ($dump == 'all') {
            $publicFields = $this->getPublicFields();
             
            foreach ($publicFields as $field) {
                if ($field['visibility'] == 1 || $field['visibility'] == 3) {
                    if (! empty($field['expr'])) {
                        $fields[$field['name']] = $field['expr'];
                    } else {
                        $fields[$field['name']] = $this->_name . '.' . $field['name'];
                    }
                }
            }
            
        } elseif ($dump == 'all_table') {
            $cols = array();
            foreach ($this->info('cols') as $col) {
                $cols[$col] = $col;
            }
            $fields = array_merge(
                $cols ,
                $fields
            );
        } elseif ($dump == 'count') {
            $fields = array(
                'result' => new Zend_Db_Expr('count(*)')
            );
        } elseif ($dump == 'length') {
            $fields = array(
                'result' => new Zend_Db_Expr("SUM(ST_Length(\"{$this->_name}\".\"the_geom\"{$geodesicStmt}))")
            );
        } elseif ($dump == 'area') {
            $fields = array(
                'result' => new Zend_Db_Expr("SUM(ST_Area(\"{$this->_name}\".\"the_geom\"{$geodesicStmt}))")
            );
        }
        
        //Add style fields
        if ($dump == 'gid_geometry_label' || $dump == 'gid_geometry') {
            if (empty($this->styleFields)) {
                $publicFields = $this->getPublicFields();
                
                foreach ($publicFields as $field) {
                    if ($field['visibility'] == 2 || $field['visibility'] == 3) {
                        if (! empty($field['expr'])) {
                            $this->styleFields[$field['name']] = $field['expr'];
                        } else {
                            $this->styleFields[$field['name']] = $this->_name . '.' . $field['name'];
                        }
                    }
                }
            }
            
            if (! empty($this->styleFields)) {
                $fields = array_merge($fields, $this->styleFields);
            }
        }
         
        if (isset($options['withIntersection'])) {
            $fields['the_geom'] = "ST_Intersection(" . $this->getAdapter()->quoteIdentifier($theGeomField) . ", {$options['withIntersection']})";
        }
        
        if (isset($options['whereIntersection'])) {
            $select->where("ST_Intersects(" . $this->getAdapter()->quoteIdentifier($theGeomField) . ", {$options['whereIntersection']})");
        }
        
        if (isset($options['whereStm'])) {
            $select->where($options['whereStm']);
        }
        
        if (! empty($options['fields'])) {
            $fields = array_merge($fields, $options['fields']);
        }
        
        $select->from($this, $fields);
        
        if ($dump == 'gid_geometry_label' || $dump == 'all' || 'count') {
            $this->joinTables($select);
        }
        
        if (! empty($options['form']) && $options['form'] instanceof Zend_Form) {
            foreach($options['form']->getElements() as $element) {
                if($element instanceof Zend_Form_Element_File) {
                    $elementName = $element->getName();
                    
                    $select->joinLeft(
                        array(
                            "files_{$elementName}" => $this->_mainSchema . '.files'
                        ), "(files_{$elementName}.id = {$this->_name}.{$elementName})", 
                        array(
                            "__filename_{$elementName}" => "files_{$elementName}.name" ,
                        )
                    ); 
                }
            }
        }
        
        //Limit
        if (! empty($options['limit'])) {
            $select->limit($options['limit']);
        }
        
        $this->_filter($select, $options);
        $this->_selectCallback($select, $options);
        
        $this->_querybuilder($select, $options);
        
        //Mask layer
        $maskingPlugin = Zend_Controller_Front::getInstance()->getPlugin('Giscuit_Controller_Plugin_Masking');
        if ($maskingPlugin != NULL && ! isset($options['withoutMask'])) {
            $maskingPlugin->whereIntersections($select, 'role_view', $this->_name);
        }
                
        return $select;
    }

    /**
     * Get an array of table fields
     * 
     * @param array $options Available options
     * @return array
     */
    public function getPublicFields (array $options = array())
    {
        $this->_joinQueue = array();
        $fieldsArr = array();
        
        if (empty($this->_settings['fields'])) {
            return array();
        }
        
        $fields = $this->_settings['fields'];
        
        //Helpers
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
        
        //Acl
        $modified = false;
        foreach ($fields as $key => $value) {
            if (! empty($value['resource']) && $isAllowed->direct($value['resource'], $value['privledge']) == false) {
                unset($fields[$key]);
                $modified = true;
            }
        }
        if ($modified == true) {
            $result = array_values($fields);
        }
        
        foreach ($fields as $field) {
            if(empty($field['name'])) {
                continue;
            }
            
            $schema = NULL;
            $newArr = array();
            
            $newArr['name'] = $field['name'];
                
            if (! empty($field['expression'])) {
                $newArr['expr'] = new Zend_Db_Expr($field['expression']);
            }
            
            $newArr['visibility'] = $field['visibility'];
            $newArr['escape'] = $field['escape'];
            $fieldsArr[$newArr['name']] = $newArr;
        }
        
        $fieldsArr = $this->_publicFieldsCallback($fieldsArr, $options);
        
        return $fieldsArr;
    }

    /**
     * Called after getPublicFields
     * 
     * @param array $fieldsArr
     * @param array $options Available options
     * @return array
     */
    protected function _publicFieldsCallback (array $fieldsArr, array $options = array())
    {
        return $fieldsArr;
    }

    /**
     * Called after getSelect to join tables to current table
     * 
     * @param Zend_Db_Select $select
     * @param array $options Available options
     * @return void
     */
    public function joinTables ($select, array $options = array())
    {
        
        if(empty($this->_settings['joins'])) {
            $this->_joinTablesCallback($select);
            return;
        }
        
        foreach ($this->_settings['joins'] as $join) {
            if(! empty( $join['table']) && ! empty($join['expression'])) {
                $select->joinLeft(
                    $this->_layersSchema . '.' . $join['table'] , 
                    $join['expression'] ,
                    NULL
                );
            }    
        }
        
        $this->_joinTablesCallback($select);
    }

    /**
     * Called after _joinTables
     * 
     * @param Zend_Db_Select $select
     * @return void
     */
    protected function _joinTablesCallback ($select)
    {}
    
    /**
     * Is called after getSelect to apply custom logic
     * 
     * @param Zend_Db_Select $select
     * @param array $options Available options
     * @return void
     */
    protected function _selectCallback ($select, array $options = array())
    {}

    /**
     * Filter select
     * 
     * @param Zend_Db_Select $select
     * @param array $options Available options
     * @return void
     */
    protected function _filter ($select, array $options = array())
    {
        if (! empty($this->_settings['filter'])) {
            $select->where($this->_settings['filter']);
        }
    }
    
    /**
     * Sets label user settings for current layer
     *
     * @param Zend_Db_Select $select
     * @param array $fields Fields
     * @return array
     */
    public function addLabelFields ($select, array $fields)
    {
        $labelFieldsArr = array();
        $separator = $this->_settings['label_separator'];
        
        //
        if(! isset($this->_settings['label']) || $this->_settings['label'] == 'f') {
            return $fields;
        }
        if(empty($this->_settings['label_fields'])) {
            return $fields;
        }
        
        //Get field options
        $publicFields = $this->getPublicFields();
        
        //Create label query string
        foreach ($publicFields as $field) {
            
            if (in_array($field['name'], $this->_settings['label_fields'])) {
                if (empty($field['expr'])) {
                    $labelFieldsArr[] = $this->getAdapter()->quoteIdentifier($this->_name . '.' . $field['name']);
                } else {
                    $labelFieldsArr[] = $field['expr'];
                }
            }
        }
        
        foreach ($labelFieldsArr as $key => $value) {
            $labelFieldsArr[$key] = "COALESCE($value::varchar, '')";
        }
        
        $labelFieldsQuery = implode("||E'{$separator}'||", $labelFieldsArr);
        
        if (empty($labelFieldsQuery)) {
            $labelFieldsQuery = 'NULL';
        }
        
        $fields['_label_'] = '(' . $labelFieldsQuery . ')';
        
        return $fields;
    
    }
    
    /**
     * Get the table default
     * 
     * This is used by the Information controller. It shows all the fields 
     * with their default field and then you click on one all the fields of the 
     * clicked row are shown.
     * 
     * @return string
     */
    public function getDefaultField ()
    {
        return $this->_defaultField;
    }

    /**
     * Get data by point coordinates and pixel size
     * 
     * @param float $x North coordinate
     * @param float $y East coordinate
     * @param float $buffer Buffer size in meters
     * @param array $options Available options:
     * <ul><li>form Zend_Form - Edit attributes form object of this layer</li>
     * <li>limit integer - Number of rows to fecth</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getByPoint ($x, $y, $buffer, array $options = array())
    {
        $config = Zend_Registry::get('config');
        
        //Compute bbox
        $x1 = $x - $buffer;
        $y1 = $y - $buffer;
        $x2 = $x + $buffer;
        $y2 = $y + $buffer;
        
        $bbox = "ST_SetSRID('BOX3D($x1 $y1, $x2 $y2)'::box3d, {$config->map->srid})";
        
        $selectOptions = array(
            'geometryAsWkt' => true
        ); 
        if(! empty($options['form'])) {
            $selectOptions['form'] = $options['form'];
        }
        
        $select = $this->getSelect('all', $selectOptions);
        
        $theGeomField = $this->getAdapter()->quoteIdentifier($this->_name . '.' . 'the_geom');
        $select->where("ST_Intersects($theGeomField, $bbox) = true");
        $select->limit($options['limit']);
        
        $result = $this->fetchAll($select);
        
        return $result;
    
    }
    
    /**
     * Get rows that have a $field LIKE $text
     * 
     * @param array $field
     * @param string $text
     * @param array $options Available options:
     * <ul><li>form Zend_Form - Edit attributes form object of this layer</li>
     * <li>bbox string - Fetch rows that are in the maximum extents of the geometry represented by a BOX2D string</li>
     * <li>limit integer - Number of rows to fecth</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getByField ($field, $text, array $options = array())
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        $selectOptions = array(
            'geometryAsWkt' => true
        ); 
        if(! empty($options['form'])) {
            $selectOptions['form'] = $options['form'];
        }
        
        $select = $this->getSelect('all', $selectOptions);

        if (! empty($options['bbox'])) {
            $this->addBbox($select, $options['bbox']);
        }
        
        $select->limit($options['limit']);
        
        //Field value
        $fields = $this->getPublicFields();

        //File
        if($options['form']->getElement($field) instanceof Zend_Form_Element_File) {
            $fieldValue = $this->getAdapter()->quoteIdentifier("files_{$field}.name");
        //Expression
        } else if (! empty($fields[$field]['expr'])) {
            $fieldValue = $fields[$field]['expr'];
        //Name
        } else {
            $fieldValue = $this->getAdapter()->quoteIdentifier($this->_name . '.' . $field);
        }
        
        //Text
        if (! empty($text)) {
            //Checkbox
            if($options['form']->getElement($field) instanceof Zend_Form_Element_Checkbox) {
                $checkedValue = $options['form']->getElement($field)->getCheckedValue();
                $uncheckedValue = $options['form']->getElement($field)->getUncheckedValue();
            
                $text = str_ireplace($translate->_($checkedValue), $checkedValue, $text);
                $text = str_ireplace($translate->_($uncheckedValue), $uncheckedValue, $text);
            }
            
            $text = '%' . str_replace(' ', '%', $text) . '%';
            $select->where("$fieldValue::varchar ILIKE ?", $text);
        }
        
        //Scale
        if (empty($this->searchScale)) {
            $scale = $this->_settings['search_scale'];
        } else {
            $scale = $this->searchScale;
        }
                
        $result = $this->fetchAll($select)->toArray();
        
        //Escape
        foreach ($result as $rowKey => $row) {
            foreach ($row as $fieldKey => $value) {
                if(! (isset($fields[$fieldKey]['escape']) && $fields[$fieldKey]['escape'] == 'f')) {
                    $result[$rowKey][$fieldKey] = htmlspecialchars($value, ENT_QUOTES);       
                } 
            }
        }

        $result = $this->formatSearchResults($result, $options['form']);
        
        return array(
            'records' => $result , 
            'scale' => $scale
        );
    }
       
    /**
     * Fetch rows that are in the maximum extents of the geometry represented by a BOX2D string
     * 
     * @param Zend_Db_Select $select
     * @param string $bbox Geometry represented by a BOX2D string
     * @return void
     */
    public function addBbox ($select, $bbox)
    {
        //the_geom field
        $theGeomField = $this->getAdapter()->quoteIdentifier($this->_name . '.' . 'the_geom');
        
        $select->where("$theGeomField && $bbox");
    }
    
    /**
     * Get area
     * 
     * @param string $geometry Geometry expression that can be represented by a field name or a string in WKT/GeoJSON format
     * @return string
     */
    protected function getArea ($geometry)
    {
        $config = Zend_Registry::get('config');
        
        if ($config->map->units == 'dd') {
            return "ST_Area($geometry, true)";
        } else {
            return "ST_Area($geometry)";
        }
    }

    /**
     * Get items
     * 
     * @param array $options Available options:
     * <ul><li>geometryAsWkt bool - Return geometry in WKT format</li>
     * <li>whereIntersection string - Fetch rows that intersect specified table name and geometry (table.the_geom)</li>
     * <li>whereId integer - Fetch row with the specified id</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getItems (array $options = array())
    {
        $theGeomField = $this->getAdapter()->quoteIdentifier($this->_name . '.the_geom');
        $gidField = $this->getPrimaryKey(array(
            'quote' => true
        ));
        
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        $fields = $this->info('cols');
                
        if (isset($options['geometryAsWkt']) && $options['geometryAsWkt'] == true) {
            $fields = array_merge($fields, array(
                'the_geom' => new Zend_Db_Expr("ST_AsText($theGeomField)")
            ));
        }
        
        $select->from($this, $fields);
        
        if (isset($options['whereId'])) {
            $select->where("$gidField = ?", $options['whereId']);
        }
        
        if (isset($options['whereIntersection'])) {
            $select->where("st_intersects($theGeomField, {$options['whereIntersection']})");
        }
 
        $result = $this->fetchAll($select)->toArray();
               
        return $result;
    }
    
    /**
     * Format information results
     * 
     * @param stdClass $itemField
     * @param string $name
     * @param mixed $value
     * @param Zend_Form $form Edit attributes form object of this layer
     * @return void
     */
    public function formatInfoResults($itemField, $name, $value, $form) {
        $translate = Zend_Registry::get('Zend_Translate');
        $module = Zend_Controller_Front::getInstance()->getRequest()->getModuleName();

        //File
        if ($form->getElement($name) instanceof Zend_Form_Element_File) {
            if ($value['id'] != NULL) {
                $url = Zend_Controller_Action_HelperBroker::getStaticHelper('url');
                $link = $url->url(array(
                    'language' => Zend_Registry::get('Zend_Translate')->getLocale() , 
                    'module' => 'default' ,
                    'controller' => 'uploads' , 
                    'action' => 'index' ,
                    'id' => $value['id']
                ), null, true);
                
                $fileParts = pathinfo($value['name']);
                
                //Image
                if (isset($fileParts['extension']) && in_array(strtolower($fileParts['extension']), Giscuit_Db_Table_Files::$imageExtensions)) {
                    //Default
                    if($module == 'tool') {
                        $itemField->html = $translate->_($name) . ": <a href=\"javascript: void(0);\" onclick=\"javascript:Giscuit.Util.photobox('{$value['name']}', '{$link}');\" title=\"{$value['name']}\">{$value['name']}</a>";
                    
                        //Mobile
                    } else if($module == 'mobile') {
                        $itemField->html = "<a href=\"javascript: void(0);\" onclick=\"javascript:Giscuit.Util.mobilePhotobox('{$value['name']}', '{$value['id']}');\" title=\"{$value['name']}\">" .  $translate->_($name) . ": {$value['name']}</a>";
                    }
                
                //File
                } else {
                    //Default
                    if($module == 'tool') {
                        $itemField->html = $translate->_($name) . ": <a href=\"{$link}\" title=\"{$value['name']}\" rel=\"external\">{$value['name']}</a>";
                    
                        //Mobile
                    } else if($module == 'mobile') {
                        $itemField->html = "<a href=\"{$link}\" title=\"{$value['name']}\" target=\"_blank\">" .  $translate->_($name) . ": {$value['name']}</a>";
                    }
                    
                }
            } else {
                $itemField->html = $translate->_($name) . ': ';
            }
        }
        
        //Checkbox
        else if ($form->getElement($name) instanceof Zend_Form_Element_Checkbox) {
            $itemField->html = $translate->_($name) . ': ' . $translate->_($value);
        } 
        
        //Default
        else {
            //Find URLs in text and make links
            $urlMatches = NULL;
            $urlPattern = "/^(http|https|ftp|ftps)\:\/\/[a-zA-Z0-9\-\.]+\.[a-zA-Z]{2,3}(\/\S*)?$/";
            if(preg_match($urlPattern, $value, $urlMatches)) {
                //Default
                if($module == 'tool') {
                    $itemField->html = $translate->_($name) . ": <a href=\"{$value}\" title=\"{$value}\" target=\"_blank\">{$value}</a>";    
                
                //Mobile
                } else if($module == 'mobile') {
                    $itemField->html = "<a href=\"{$value}\" title=\"{$value}\" target=\"_blank\">" . $translate->_($name) . ": {$value}</a>";
                }
                
            } else {
                $itemField->html = $translate->_($name) . ': ' . $value;    
            }
            
        }
    }
    
    /**
     * Format search results
     * 
     * @param Zend_Db_Table_Rowset $rows
     * @param Zend_Form $form Edit attributes form object of this layer
     * @return Zend_Db_Table_Rowset
     */
    public function formatSearchResults($rows, $form) {
        $translate = Zend_Registry::get('Zend_Translate');
        $module = Zend_Controller_Front::getInstance()->getRequest()->getModuleName();
        
        foreach($form->getElements() as $element) {
            //File
            if($element instanceof Zend_Form_Element_File) {
                $elementName = $element->getName();

                foreach($rows as $key => $value) {
                    if (isset($rows[$key][$elementName]) && $rows[$key][$elementName]) {
                        $rows[$key][$elementName] = array(
                            'id' => $rows[$key][$elementName] ,
                            'name' => $rows[$key]['__filename_' . $elementName]
                        );
                        $url = Zend_Controller_Action_HelperBroker::getStaticHelper('url');
                        $fileLink = $url->url(array(
                            'language' => Zend_Registry::get('Zend_Translate')->getLocale() , 
                            'module' => 'default' ,
                            'controller' => 'uploads' , 
                            'action' => 'index' ,
                            'id' => $rows[$key][$elementName]['id']
                        ), null, true);
                        
                        $fileParts = pathinfo($rows[$key][$elementName]['name']);
                        
                        //Image
                        if (isset($fileParts['extension']) && in_array(strtolower($fileParts['extension']), Giscuit_Db_Table_Files::$imageExtensions)) {
                            //Default
                            if($module == 'tool') {
                                $rows[$key][$elementName] = "<a href=\"javascript: void(0);\" onclick=\"Giscuit.Util.photobox('{$rows[$key][$elementName]['name']}', '{$fileLink}');\" title=\"{$rows[$key][$elementName]['name']}\">{$rows[$key][$elementName]['name']}</a>";
                            } else if ($module == 'mobile') {
                                //Mobile
                                $rows[$key][$elementName] = array(
                                        'type' => 'image' ,
                                        'value' => "<a href=\"javascript: void(0);\" onclick=\"javascript:Giscuit.Util.mobilePhotobox('{$rows[$key][$elementName]['name']}', '{$rows[$key][$elementName]['id']}');\" title=\"{$rows[$key][$elementName]['name']}\">{$elementName}: {$rows[$key][$elementName]['name']}</a>"
                                );
                            }
                            
                        //File    
                        } else {
                            //Default
                            if($module == 'tool') {
                                $rows[$key][$elementName] = "<a href=\"{$fileLink}\" title=\"{$rows[$key][$elementName]['name']}\">{$rows[$key][$elementName]['name']}</a>";
                            
                                //Mobile
                            } else if ($module == 'mobile') {
                                $rows[$key][$elementName] = array(
                                        'type' => 'file' ,
                                        'value' => $rows[$key][$elementName]['name'],
                                        'url' => $fileLink
                                );
                            }
                            
                        }
                        
                    } else {
                        $rows[$key][$elementName] = '';
                    }
                    unset($rows[$key]['__filename_' . $elementName]);
                }
            }
            
            //Checkbox
            else if($element instanceof Zend_Form_Element_Checkbox) {
                $elementName = $element->getName();

                foreach($rows as $key => $value) {
                    if(isset($rows[$key][$elementName]) && $rows[$key][$elementName]) {
                        $rows[$key][$elementName] = $translate->_($rows[$key][$elementName]);
                    }
                }    
            } 
            
            //Default
            else {
                                            
            }
             
        }
        
        return $rows;
    }
    
    /**
     * Query builder
     * 
     * @param Zend_Db_Select $select
     * @param array $options Available options:
     * <ul><li>querybuilder array - Query builder settings</li></ul>
     * @return void
     */
    protected function _querybuilder ($select, array $options = array())
    {
        $settings = Zend_Registry::get('settings');
        $config = Zend_Registry::get('config');
        
        //
        if(! empty($options['querybuilder'])) {
            $querybuilder = $options['querybuilder'];
        } elseif(! empty($settings->querybuilder)) {
            $querybuilder = $settings->querybuilder;
                        
            if($querybuilder['save'] == 'f') {
                return;
            }
        } else {
            $querybuilder = NULL;
        }

        //
        if(empty($querybuilder)) {
            return;
        } 
        
        //
        if(empty($querybuilder[$this->_settings['id']])) {
            return;
        }
        $querybuilder = $querybuilder[$this->_settings['id']];
                
        $joinTables = array();
        
        $conditions = $querybuilder['conditions'];
        $match = $querybuilder['match'];
        
        if($match == 1) {
            $whereStr = 'where';
        } else if($match == 2) {
            $whereStr = 'orWhere';
        }
        
        foreach($conditions as $condition) {
            
            $fieldName = $geometryTableA = $geometryTableB = NULL;
            
            if($this->_settings['table'] != $condition['table']) {
                $joinTables[] = $condition['table'];
            }
            
            if($condition['field']['name'] != 'the_geom') {
                if (empty($condition['field']['expr'])) {
                    $fieldName = $this->getAdapter()->quoteIdentifier(
                        $condition['table'] . '.' .  $condition['field']['name'] 
                    );
                } else {
                    $fieldName = new Zend_Db_Expr($condition['field']['expr']); 
                }
                
                //IS NULL, IS NOT NULL
                if($condition['operator'] == 'IS NULL' || $condition['operator'] == 'IS NOT NULL') {
                    $select->$whereStr("{$fieldName} {$condition['operator']}");
                    
                //ILIKE, NOT ILIKE
                } else if($condition['operator'] == 'ILIKE' || $condition['operator'] == 'NOT ILIKE') {
                    $select->$whereStr("{$fieldName}::varchar {$condition['operator']} ?" , $condition['value0']);
                    
                //IN, NOT IN
                } else if($condition['operator'] == 'IN' || $condition['operator'] == 'NOT IN') {
                    $inValues = explode(',', $condition['value0']);
                    foreach ($inValues as $key => $inValue) {
                        $inValues[$key] = $this->getAdapter()->quote($inValues[$key]);
                    }
                    $inExpr = new Zend_Db_Expr('(' . implode(',', $inValues)  . ')');
                                        
                       $select->$whereStr("{$fieldName} {$condition['operator']} ?" , $inExpr);
                } else {
                    $select->$whereStr("{$fieldName} {$condition['operator']} ?" , $condition['value0']);
                }            
            } else {
                if($config->map->units == 'dd') {
                    $useSpheroid = ',true';        
                } else {
                    $useSpheroid = '';    
                }
                
                $geometryTableA = $this->getAdapter()->quoteIdentifier(
                    //$this->_settings['table'] . '.' . 'the_geom'
                    $condition['table'] . '.' . 'the_geom'
                );
                
                if($condition['funct'] == 'ST_IsValid' || $condition['funct'] == 'NOT ST_IsValid') {
                    $select->$whereStr("{$condition['funct']}($geometryTableA)");
                       
                } else if (in_array($condition['funct'], array("ST_Area", "ST_Length", "ST_Perimeter"))) {
                    $select->$whereStr("{$condition['funct']}($geometryTableA $useSpheroid) {$condition['operator']} ?" , $condition['value0']);
                    
                } else if($condition['funct'] == 'ST_DWithin') {
                    if(! empty($condition['geometryTable'])) {
                        $joinTables[] = $condition['geometryTable'];
                
                        $geometryTableB = $this->getAdapter()->quoteIdentifier(
                            $condition['geometryTable'] . '.' . 'the_geom'
                        );
                    }
                    
                    $select->$whereStr("{$condition['funct']}($geometryTableA, $geometryTableB, {$condition['value0']} $useSpheroid)");
                    
                } else {
                    $joinTables[] = $condition['geometryTable'];
                
                    $geometryTableB = $this->getAdapter()->quoteIdentifier(
                        $condition['geometryTable'] . '.' . 'the_geom'
                    ); 
                    
                    $select->$whereStr("{$condition['funct']}($geometryTableA, $geometryTableB)");
                    
                }
                
            }
        }
         
        $joinTables = array_unique($joinTables);
                
        foreach ($joinTables as $table) {
            if ($this->_name == $table) {
                continue;    
            }
            
            $select->joinLeft(
                $this->_layersSchema . '.' . $table ,
                '(TRUE = TRUE)',
                NULL 
            );
        }
        
        $this->_querybuilderCallback($select);
    }
    
    /**
     * Called after _querybuilder
     * 
     * @param Zend_Db_Select $select
     * @param array $options Available options
     * @return void
     */
    protected function _querybuilderCallback ($select, array $options = array())
    {
    }
    
    /**
     * Fetch unique value of a column
     * 
     * @param string $field Column from which to fetch values
     * @param array $optionsAvailable options:
     * <ul><li><li>limit integer - Number of rows to fecth</li></ul>
     * @return array $fields Array of unique values of a column
     */
    public function getUniqueColumnValues($field, array $options = array())
    {
        $publicFields = $this->getPublicFields();
        
        if (! isset($publicFields[$field])) {
            throw new Giscuit_Exception("Field {$field} does not exist");
        }

        $select = $this->select();
        
        if (isset($publicFields[$field]['expr'])) {
            $select->from($this, array('name' => new Zend_Db_Expr($publicFields[$field]['expr'])));
            $select->where("{$publicFields[$field]['expr']} IS NOT NULL");
        } else {
            $select->from($this, array('name' => $field));
            $select->where("{$field} IS NOT NULL");
        }
        
        $select->distinct();
        
        $this->joinTables($select);
        
        if (isset($options['limit'])) {
            $select->limit($options['limit']);
        }
        
        $fields = array();
        $result = $this->fetchAll($select);
        foreach ($result as $value) {
            $fields[] = $value->name;
        }
        
        return $fields;
    }
    
    /**
     * Get maximum and minimum value of a field
     * 
     * @param Zend_Db_Table_Row $field Field
     * @return Zend_Db_Table_Rowset
     */
    public function getMaxAndMinValues ($field) 
    {   
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        if (! empty($field['expression'])) {
            $select->from($this, array(
                'maximum' => new Zend_Db_Expr("MAX({$field['expression']})"),
                'minimum' => new Zend_Db_Expr("MIN({$field['expression']})")
            ));
        } else {
            $select->from($this, array(
                'maximum' => new Zend_Db_Expr("MAX({$field['name']})"),
                'minimum' => new Zend_Db_Expr("MIN({$field['name']})")
            ));
        }

        $result = $this->fetchAll($select);
        
        return $result;
    }
    
    /**
     * Get maximum and minimum value of a field
     * 
     * @param Zend_Db_Table_Row $field Field
     * @param array $options Available options:
     * <ul><li><li>limit integer - Number of rows to fecth</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getRandomValues ($field, array $options = array()) 
    {
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        if (! empty($field['expression'])) {
            $select->from($this, array(
                $field['name'] => new Zend_Db_Expr($field['expression'])
            ));
        } else {
            $select->from($this, $field['name']);
        }
        
        $select->order('RANDOM()');
        if (isset($options['limit'])) {
            $select->limit($options['limit']);
        } else {
            $select->limit('2000');
        }

        $result = $this->fetchAll($select);
        
        return $result;
    }
    
    /**
     * Add item
     * 
     * @param Zend_Db_Table_Row $row New row
     * @param Zend_Form $form Form object populated with request data
     * @return integer $pkey Row id
     */
    public function addItemForEditCallback($row, $form = NULL)
    {
        $pkey = $row->save();
        
        return $pkey;
    }
    
    /**
     * Update item
     * 
     * @param Zend_Db_Table_Row $row Updated row
     * @param Zend_Form $form Form object populated with request data
     * @return bool Update/don't update item
     */
    public function updateItemForEditCallback($row, $form = NULL, $oldRow)
    {
        $row->save();
        
        return true;
    }
    
    /**
     * Delete item
     * 
     * @param integer $gid Row id
     * @return bool Delete/don't delete item
     */
    public function deleteItemForEditCallback($gid)
    {
        return true;
    }
    
    /**
     * Get item for edit
     * 
     * @param Zend_Db_Table_Row $row Updated row
     * @param Zend_Form $form Form object populated with request data
     * @return Zend_Db_Table_Row $row Updated row
     */
    public function getItemForEditCallback($row, $form = NULL)
    {
        return $row;
    }
    
    /**
     * Get item for add
     *
     * @param Zend_Form $form Form object populated with request data
     * @return void
     */
    public function getItemForAddCallback($form = NULL)
    {
    }
    
    /**
     * Update geometry
     * 
     * @param string $geometry Geometry represented as WKT
     * @return string $geometry Geometry represented as WKT
     */
    public function parseGeometryBeforeEdit($geometry)
    {
        return $geometry;
    }

    /**
     * Get extent
     *
     * @return string
     */
    public function getExtent ()
    {
        $theGeomField = $this->getAdapter()->quoteIdentifier($this->_name . '.' . 'the_geom');

        $select = $this->select();
        $select->setIntegrityCheck(false);

        $select->from($this, array(
            'extent' => new Zend_Db_Expr("ST_Extent({$theGeomField})")
        ));

        $result = $this->fetchRow($select)->extent;
        $result = substr($result, 4, -1);
        $result = str_replace(' ', ',', $result);

        return $result;
    }
}
