<?php

/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: History.php 3284 2013-01-23 09:32:49Z vlad.bruma $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Layer_Abstract
 */
require_once 'Giscuit/Db/Table/Layer/Abstract.php';

/**
 * History class for table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_Layer_History extends Giscuit_Db_Table_Layer_Abstract
{

    /**
     * History fields
     * 
     * @var array
     */
    protected $_historyFields = array(
        'hgid' , 
        'h_id' , 
        'hstatus'
    );
    
    /**
     *  Is layer in history mode
     *  
     * @var bool
     */
    public $historyEnabled = false;
    
    /**
     * Constructor
     *
     * @param array $options Available options
     * @return void
     */
    public function __construct (array $options = array())
    {

        $config = Zend_Registry::get('config');
                
        if(isset($options['withHistory']) && $options['withHistory'] == true) {
            $this->historyEnabled = true;
        }
   
        if($this->historyEnabled == true) {
            $this->_primary = 'hgid';
        
            if(empty($this->_schema)) {
                $this->_schema = $config->database->schema . '_history';
                $this->_mainSchema = $config->database->schema;
            }
            
            $this->_layersSchema = $config->database->schema . '_layers';
            $this->_historySchema = $config->database->schema . '_history';    
        }
        
        parent::__construct($options);

    }
    
    /**
     * Activate temporary changes 
     * 
     * @param integer $hId History id
     * @param Giscuit_Db_Table_Layer_Abstract $table Layer table
     * @return void
     */
    public function activateHistory ($hId, $table)
    {
        $hidField = $this->getAdapter()->quoteIdentifier($this->_name . '.h_id');
        $pkeyName = $table->getPrimaryKey(array(
            'name' => true
        ));
        
        $select = $this->select();
        $select->from($this);
        $select->where("$hidField = ?", $hId);
        
        $hRows = $this->fetchAll($select);
        
        foreach ($hRows as $hRow) {
            
            if ($hRow->hstatus == 'C' || $hRow->hstatus == 'U') {
                
                if (! empty($hRow->$pkeyName)) {
                    //Edit
                    $rows = $table->find($hRow->$pkeyName);
                    $row = $rows->current();
                    
                    if (! empty($row)) {
                        $row = $this->_populateRow($hRow, $row);
                        $row->save();
                    }
                
                } else {
                    //Add
                    $row = $table->createRow();
                    $row = $this->_populateRow($hRow, $row);
                    
                    //Save new row and set new gid in the history row
                    $hRow->$pkeyName = $row->save();
                    
                    //Save history row
                    $hRow->save();
                }
                //Delete
            } elseif ($hRow->hstatus == 'D') {
                $rows = $table->find($hRow->$pkeyName);
                $row = $rows->current();
                if (! empty($row)) {
                    $row->delete();
                }
            }
        
        }
        
        $history = new Giscuit_Db_Table_History();
        $history->setActive($hId, true);
    
    }

    /**
     * Populate row
     * 
     * @param Zend_Db_Table_Row_Abstract $hRow History layer row
     * @param  Zend_Db_Table_Row_Abstract $row Layer row
     * @return  Zend_Db_Table_Row_Abstract
     */
    protected function _populateRow ($hRow, $row)
    {
        $hRowArr = $hRow->toArray();
        
        foreach ($hRowArr as $key => $column) {
            if (! in_array($key, $this->_historyFields)) {
                $row->$key = $column;
            }
        }
        
        return $row;
    }

    /**
     * Clear temporary changes 
     * 
     * @param integer $hId
     * @param integer $userId
     * @return void
     */
    public function clearHistory ($hId, $userId)
    {
        $select = $this->select();
        $select->setIntegrityCheck(false);
        $select->from($this, array(
            'hgid' => $this->getPrimaryKey()
        ));
        
        $hidField = $this->getAdapter()->quoteIdentifier($this->_name . '.h_id');
        $hgidField = $this->getPrimaryKey(array(
            'quote' => true
        ));
        
        $select->where("$hidField = ?", $hId);
        $select->where('history.fetch_user_id = ?', $userId);
        
        //TODO: Replace with inner join and remove where
        $select->joinLeft(array(
            'history' => $this->_mainSchema . '.history'
        ), "(history.id = $hidField AND history.active = 'f')", NULL);
        
        $rows = $this->fetchAll($select);
        
        //Prepare delete query
        $delQuery = array();
        foreach ($rows as $row) {
            $delQuery[] = $this->getAdapter()->quoteInto("$hgidField = ?", $row->hgid);
        }
        $delQuery = implode(' OR ', $delQuery);
        
        if (! empty($delQuery)) {
            $this->delete($delQuery);
        }
    
    }

    /**
     * Apply custom history logic. By default will show the latest version of 
     * rows with out the deleted ones in a given interval specified by   
     * $settings->historyDateBegin and $settings->historyDateEnd
     *  
     * @param Zend_Db_Select $select
     * @param array $options Available options:
     * <ul><li>whereHistoryId integer - Show only temporary changes for the specified value</li>
     * <li>whereRawHistory bool - Show all changes</li></ul>
     * @return void
     */
    protected function _selectCallback ($select, array $options = array())
    {
        parent::_selectCallback($select, $options);
        
        if($this->historyEnabled == false) {
            return;
        } 
        
        $config = Zend_Registry::get('config');
                
        //FIXME: Primary key name "gid" is static
        $gidField = 'gid';
        $hidField = $this->getAdapter()->quoteIdentifier($this->_name . '.h_id');
        $hstatusField = $this->getAdapter()->quoteIdentifier($this->_name . '.hstatus');
        $hgidField = $this->getPrimaryKey(array(
            'quote' => true
        ));
        
        if (! empty($options['whereHistoryId'])) {
            $select->where("$hidField = ?", $options['whereHistoryId']);
            $select->where('globhist.active = ?', 'f');
            
            $select->order($this->_name . ".hstatus asc");

        } elseif(isset($options['whereRawHistory']) && $options['whereRawHistory'] == true) {
            $this->_historyInterval($select);
        } else {
            
            //History, compare mode off
            $subSelect = $this->select();
            
            $subSelect->from($this, array(
                'gid' => $gidField , 
                'hgid' , 
                'hstatus' ,
                
            ));
            $this->_historyInterval($subSelect);
            $subSelect->order('history.activate_date desc');
            
            //Where in Subselect
            $select->where("$hgidField IN (SELECT {$config->database->schema}.first(hlayer.hgid) FROM ({$subSelect}) as hlayer GROUP BY hlayer.gid)");
            
            //Do not show deleted geometries
            $select->where("$hstatusField != 'D'");
        }
    }
    
    /**
     * Fetch rows that match an interval specified by $settings->historyDateBegin
     * and $settings->historyDateEnd
     * 
     * @param Zend_Db_Select $select
     * @return void
     */
    protected function _historyInterval($select) {
        $settings = Zend_Registry::get('settings');
        $hidField = $this->getAdapter()->quoteIdentifier($this->_name . '.h_id');

        $select->joinInner(array(
            'history' => $this->_mainSchema . '.history'
        ), "(history.id = $hidField AND history.active = 't')", NULL);        

        //History interval
        if (! empty($settings->historyDateBegin)) {
            $select->where('history.activate_date > ?', new Zend_Db_Expr("to_timestamp('{$settings->historyDateBegin}', '" . GT_DATE_STYLE_DB . "')"));
        }
                
        if (! empty($settings->historyDateEnd)) {
            $select->where('history.activate_date < ?', new Zend_Db_Expr("to_timestamp('{$settings->historyDateEnd}', '" . GT_DATE_STYLE_DB . "') + interval '1 day' - interval '1 second'"));
        }    
    }

    /**
     * Add row with history
     * 
     * @param array $data Data
     * @param array $options Available options:
     * <ul><li>geometry string - Geometry in WKT format</li></ul>
     * @return array
     */
    public function addItemForHistory ($data, array $options = array())
    {
        $newRow = $this->createRow();
        
        foreach ($data as $key => $value) {
            $newRow->$key = $value;
        }
        
        return array(
            'object' => $newRow , 
            'table' => $this , 
            'geometry' => $options['geometry']
        );
    }

    /**
     * Add history fields (created_by, verified_by, changed_date)
     *
     * @param array $fieldsArr
     * @param array $options Available options
     * @return array
     */
    protected function _publicFieldsCallback (array $fieldsArr, array $options = array())
    {
        
        $fieldsArr = parent::_publicFieldsCallback($fieldsArr);
        
        if($this->historyEnabled == false) {
             return $fieldsArr;
        }
        
        $fieldsArr[] = array(
            'name' => 'created_by' , 
            'visibility' => 1 , 
            'expr' => new Zend_Db_Expr("created_users.fname||' '||created_users.lname") , 
        );
        $fieldsArr[] = array(
            'name' => 'verified_by' , 
            'visibility' => 1 , 
            'expr' => new Zend_Db_Expr("verified_users.fname||' '||verified_users.lname") , 
        );
        $fieldsArr[] = array(
            'name' => 'changed_date' , 
            'visibility' => 1 , 
            'expr' => new Zend_Db_Expr("to_char(globhist.activate_date, '" . GT_DATETIME_STYLE_DB . "')") , 
        );
        
        return $fieldsArr;
        
    }
    
    /**
     * Join history tables
     *
     * @param Zend_Db_Select $select
     * @return void
     */
    protected function _joinTablesCallback ($select)
    {
        
        parent::_joinTablesCallback($select);
        
        if($this->historyEnabled == false) {
            return;
        }
        
        $hidField = $this->getAdapter()->quoteIdentifier($this->_name . '.h_id');
        
        $select->joinLeft(array(
            'globhist' => $this->_mainSchema . '.history'
        ), "($hidField = globhist.id)", array(
            'changed_date' => new Zend_Db_Expr("to_char(globhist.activate_date, '" . GT_DATETIME_STYLE_DB . "')")
        ));
        
        $select->joinLeft(array(
            'created_users' => $this->_mainSchema . '.users'
        ), "(globhist.fetch_user_id = created_users.id)", array(
            'created_by' => new Zend_Db_Expr("created_users.fname||' '||created_users.lname")
        ));
        
        $select->joinLeft(array(
            'verified_users' => $this->_mainSchema . '.users'
        ), "(globhist.activate_user_id = verified_users.id)", array(
            'verified_by' => new Zend_Db_Expr("verified_users.fname||' '||verified_users.lname")
        ));
    
    }

    /**
     * Add a history row from the row's current state and return it's id
     * 
     * @param integer $gid Row id
     * @param Giscuit_Db_Table_Layer_Abstract $table Layer table
     * @param array $options <ul>Available options:
     * <li>hstatus string - Type of the modification C(create) U(update) D(delete)</li></ul>
     * @return integer
     */
    public function addFromParent ($gid, $table, array $options = array())
    {
        
        $select = $table->select();
        $select->setIntegrityCheck(false);
        $fields = $table->info('cols');
        
        foreach ($fields as $key => $value) {
            if (in_array($value, $this->_historyFields)) {
                unset($fields[$key]);
            }
        }
        
        $pkeyField = $table->getPrimaryKey(array(
            'quote' => true
        ));
        
        //Get insert fields query
        $insertFields = array_merge($this->_historyFields, array_values($fields));
        foreach ($insertFields as $key => $value) {
            $insertFields[$key] = $this->getAdapter()->quoteIdentifier($insertFields[$key]);
        }
        $insertFields = implode(', ', $insertFields);
               
        $hgid = $this->getAdapter()->nextSequenceId($this->info('sequence'));
                
        $fields = array_merge(array(
            'hgid' => new Zend_Db_Expr($hgid) , 
            'h_id' => new Zend_Db_Expr($options['hId']) , 
            'hstatus' => new Zend_Db_Expr("'{$options['hstatus']}'")
        ), $fields);
        
        $select->from($table, $fields);
        $select->where($pkeyField . ' = ?', $gid);
        
        $query = "INSERT INTO \"{$this->_schema}\".\"{$this->_name}\" ($insertFields) $select";

        $table->getAdapter()->query($query);

        return $hgid;
    }

    /**
     * Get all changes of a specified row
     * 
     * @param integer $gid Row id
     * @return mixed
     */
    public function getHistoryVersions($gid) 
    {
        
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        //FIXME: Primary key name "gid" is static
        $gidField = 'gid';
        $hidField = $this->getAdapter()->quoteIdentifier($this->_name . '.h_id');
        $hstatusField = $this->getAdapter()->quoteIdentifier($this->_name . '.hstatus');
        
        $fields = array(
            'h_id' ,
        );
        
        $select->from($this, $fields);
        
        //Do not show deleted geometries
        $select->where("$hstatusField != 'D'");
        
        //Join date field
        $select->joinLeft(array(
            'globhist' => $this->_mainSchema . '.history'
        ), "($hidField = globhist.id)", array(
            'hdate' => new Zend_Db_Expr("to_char(globhist.activate_date, '" . GT_DATETIME_STYLE_DB . "')")
        ));
                
        $select->where($this->getAdapter()->quoteIdentifier($this->_name . '.' . $gidField) . ' = ?', $gid);
        
        //Order
        $select->order('globhist.activate_date desc');
        
        $result = $this->fetchAll($select);
        
        return $result;
        
    }
}
