<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Users.php 3547 2014-05-21 15:03:33Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Class for users table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_Users extends Giscuit_Db_Table_Abstract
{

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'users';

    /**
     * Get items
     * 
     * @param array $options Available options:
     * <ul><li>whereId integer - Fetch rows that match the id</li>
     * <li>whereNameLike string - Get elements with similar name</li>
     * <li>whereAccount string - Fetch rows that match the account field</li>
     * <li>whereIdsIn string - Get elements whose ids are in the string</li>
     * <li>limit integer - Number of results to get</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getItems (array $options = array())
    {
        $config = Zend_Registry::get('config');
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        $fields = array(
            'id' , 
            'active' ,
            'fname' , 
            'lname' , 
            'account' , 
            'group' , 
            'language' , 
            'snap' , 
            'lastlogin_date' => "extract(epoch from lastlogin_date)" ,
            'expiration_date' => "extract(epoch from expiration_date)" ,
            'ip',
            'tooltip',
            'timezone',
            'email',
            'email_active',
            'email_secret',
            'touch_offset',
            'tables',
            'netcheck'
        );
        
        if(isset($options['withIdentity'])) {
            $fields['identity'] = new Zend_Db_Expr($config->userIdentityExpr);
        }
        
        $select->from($this, $fields);
        
        //
        $this->whereId($select, $options);
        $this->whereIdentityLike($select, $options);
        $this->whereActive($select, $options);
        
        if(isset($options['whereAccount'])) {
            $select->where($this->_name . '.account = ?', $options['whereAccount']);
        }
        
        if(isset($options['whereIdsIn'])) {
            $select->where($this->_name . ".id IN ({$options['whereIdsIn']})");
        }
        
        if(isset($options['limit'])) {
            $select->limit($options['limit']);
        }
        
        //
        $select->order($this->_name . '.id desc');
        
        $result = $this->fetchAll($select);
        
        return $result;
    }
    
    /**
     * Fetch rows that match the text
     * 
     * @param Zend_Db_Select $select
     * @param array $options Available options:
     * <ul><li>whereIdentityLike string - If option is set selects rows that match the text</li></ul>
     * @return void
     */
    public function whereIdentityLike ($select, array $options = array())
    {
        if (! isset($options['whereIdentityLike'])) {
            return;
        }
        
        $config = Zend_Registry::get('config');
        
        $query = '%' . str_replace(' ', '%', strtolower(trim($options['whereIdentityLike']))) . '%';
        
        $select->where($config->userIdentityExpr . ' ILIKE ?', $query);
    }

    /**
     * Add item
     * 
     * @param Zend_Form $data A form object populated with request data
     * @param array $options Available options:
     * <ul><li>emailSecret string - Set the email secret to current user's account</li></ul>
     * @return integer The id of the new element
     */
    public function addItem ($data, array $options = array())
    {
        $config = Zend_Registry::get('config');
        $newRow = $this->createRow();
        
        //$newRow->username = $data->getValue('username');
        $newRow->password = md5($data->getValue('password') . $config->salt);
        $newRow->account = $data->getValue('account');
        $newRow->fname = $data->getValue('fname');
        $newRow->lname = $data->getValue('lname');
        $newRow->group = $config->signup->resource;
        
        $newRow->signup_date = new Zend_Db_Expr('now()');
        
        if($config->signup->activate == 1 || $config->signup->activate == 2) {
            $newRow->active = 'f';    
        } else if ($config->signup->activate == 3) {
            $newRow->active = 't';
        }
        
        $newRow->timezone = $data->getValue('timezone');
        
        $newRow->language = Zend_Registry::get('Zend_Translate')->getLocale();
        
        $newRow->email = $data->getValue('account');
        
        if(isset($options['emailSecret'])) {
            $newRow->email_secret = $options['emailSecret'];
        }
        
        $result = $newRow->save();
        
        //eventUserCreate
        Giscuit_Controller_Plugin_Abstract::call('eventUserCreate', 
            array(
                'row' => $newRow,
                'password' => $data->getValue('password')
            )
        );
        
        return $result;
    }

    /**
     * Set lastlogin_date to current date 
     * 
     * @param integer $userId User id
     * @return void 
     */
    public function updateLastLoginDate ($userId)
    {
        $rows = $this->find($userId);
        $row = $rows->current();
        $row->lastlogin_date = new Zend_Db_Expr('now()');
        
        $row->save();
        
        //eventUserUpdate
        Giscuit_Controller_Plugin_Abstract::call('eventUserUpdate',
            array(
                'row' => $row
            )
        );
    }

    /**
     * Update item
     * 
     * @param integer $id The id of the row to edit 
     * @param Zend_Form $data A form object populated with request data
     * @return void
     */
    public function editItem ($id, $data)
    {
        $settings = Zend_Registry::get('settings');
        
        $rows = $this->find($id);
        $row = $rows->current();
        
        //Set new values
        $row->language = $data->getValue('giscuit_language');
        $row->snap = $data->getValue('snap') == 't' ? 't' : 'f';
        $row->tooltip = $data->getValue('tooltip') == 't' ? 't' : 'f';
        $row->timezone = $data->getValue('timezone');
        $row->touch_offset = $data->getValue('touch_offset');
        $row->netcheck = $data->getValue('netcheck');
        
        //Save
        $row->save();
        
        //eventUserUpdate
        Giscuit_Controller_Plugin_Abstract::call('eventUserUpdate',
            array(
                'row' => $row
            )
        );
    }
    
    /**
     * Update item from admin panel
     * 
     * @param integer $id The id of the row to edit 
     * @param Zend_Filter_Input $data A Zend_Filter_Input object populated with request data
     * @return integer The id of the element
     */
    public function adminEditItem ($id, $data)
    {
        $rows = $this->find($id);
        $row = $rows->current();
        
        //Get column
        $column = $data->column;
        
        if (empty($data->data)) {
            $data->data = NULL;
        }
        
        $row->$column = $data->data;

        $result = $row->save();
        
        //eventUserUpdate
        Giscuit_Controller_Plugin_Abstract::call('eventUserUpdate',
            array(
                'row' => $row
            )
        );
        
        return $result;
    }

    /**
     * Update language value
     * 
     * @param integer $id The id of the row to edit 
     * @param string $language Language
     * @return void
     */
    public function editLanguage ($id, $language)
    {
        $rows = $this->find($id);
        $row = $rows->current();
        
        $row->language = $language;
        
        $row->save();
        
        //eventUserUpdate
        Giscuit_Controller_Plugin_Abstract::call('eventUserUpdate',
            array(
                'row' => $row
            )
        );
    }
    
    /**
     * Get password
     * 
     * @param integer $userId User id
     * @return string Users password in md5
     */
    public function getPassword ($userId)
    {
        $rows = $this->find($userId);
        $row = $rows->current();
        
        return $row->password;
    }
    
    /**
     * Get language
     * 
     * @param integer $userId User id
     * @return string Users language
     */
    public function getLanguage ($userId)
    {
        $rows = $this->find($userId);
        $row = $rows->current();
        
        return $row->language;
    }

    /**
     * Update password
     * 
     * @param integer $userId User id
     * @param string $newPassword New password
     * @return void
     */
    public function setPassword ($userId, $newPassword)
    {
        $config = Zend_Registry::get('config');
        
        $rows = $this->find($userId);
        $row = $rows->current();
        
        $row->password = md5($newPassword . $config->salt);
        
        $row->save();
        
        //eventUserUpdate
        Giscuit_Controller_Plugin_Abstract::call('eventUserUpdate',
            array(
                'row' => $row,
                'password' => $newPassword
            )
        );
    }
    
    /**
     * Check if current user owns the table
     * 
     * @param string $tableName Name of the table
     * @return void
     */
    public function checkCurrUserOwnTable($tableName) 
    {
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
        
        if (! $isAllowed->direct('admin')) {
            
            $select = $this->select();
            $select->from($this, array('id'));
            $select->where('id = ?', Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct());
            $select->where("tables LIKE '%{$tableName},%'");
                    
            $result = $this->fetchRow($select);

            if (count($result) == 0) {
                throw new Giscuit_Exception("Invalid table name");
            }
        }
    }
    
    /**
     * Get all table names that belong to users
     * 
     * @return string All users tables
     */
    public function getTables() 
    {
        $select = $this->select();
        
        $fields = array(
            'text' => new Zend_Db_Expr("array_to_string(array_agg(',' || COALESCE(tables, '') ||'\t'|| id ||'\t'|| account), '\n')")
        );
        
        $select->from($this, $fields);

        $result = $this->fetchRow($select);
        
        return $result->text . "\n";
    }

}