<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Abstract.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Map
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Map_Abstract
 */
require_once 'Giscuit/Map/Abstract.php';

/**
 * Class for abstract mapserver layer object
 *
 * @category   Giscuit
 * @package    Giscuit_Map
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */
abstract class Giscuit_Map_Layer_Abstract extends Giscuit_Map_Abstract
{
    /**
     * Settings
     *
     * @var array
     */
    protected $_settings;

    /**
     * Class constructor
     *
     * @param mixed $map Map object
     * @param array $settings Settings
     * @return void
     */
    public function __construct ($map, $settings)
    {
        $this->_settings = $settings;
        
        if(empty($this->_settings['class'])) {
            throw new Giscuit_Exception('Layer name is empty');
        }
        
        $this->_obj = ms_newLayerObj($map->getMsObj());
        $this->parent = $map;
                
        //Set layer type
        if ($this instanceof Giscuit_Map_Layer_Point) {
            $this->set('type', MS_LAYER_POINT);
        } elseif($this instanceof Giscuit_Map_Layer_Line) {
            $this->set('type', MS_LAYER_LINE);
        } elseif($this instanceof Giscuit_Map_Layer_Polygon) {
            $this->set('type', MS_LAYER_POLYGON);
        } elseif($this instanceof Giscuit_Map_Layer_Raster || 
            $this instanceof Giscuit_Map_Layer_Wms) {
                
            $this->set('type', MS_LAYER_RASTER);
        }
        
        $this->set('name', $this->_settings['class']);
        $this->set('status', MS_ON);
        
        $this->parent->children[$this->name] = $this;

        //Tolerance
        if (! empty($this->_settings['tolerance'])) {
            $this->set('tolerance',$this->_settings['tolerance']);
        }
        
        //Settings
        if (! empty($this->_settings['minscaledenom'])) {
            $this->set('minscaledenom', $this->_settings['minscaledenom']);
        }
        
        if (! empty($this->_settings['maxscaledenom'])) {
            $this->set('maxscaledenom', $this->_settings['maxscaledenom']);
        }
        
        if (! empty($this->_settings['opacity'])) {
            $this->set('opacity', $this->_settings['opacity']);
        }

    }

    /**
     * Set settings
     * 
     * @param array $settings Settings
     * @return void
     */
    public function setSettings ($settings)
    {
        $this->_settings = $settings;
    }
    
    /**
     * Get settings
     * 
     * @return array
     */
    public function getSettings ()
    {
        return $this->_settings;
    }
    
    /**
     * Parse tilecache params
     * 
     * @param array $result
     * @param array $options <ul>Available options:
     * <li>class string - </li>
     * <li>transparent bool - </li>
     * <li>format string - </li></ul>
     * @return array
     */
    public static function parseTilecacheParams($result, array $options = array()) {
        return $result;
    }
    
    /**
     * Returns a class from the layer given an index
     * 
     * @param integer $index Index value (0 = first class)
     * @return Giscuit_Map_Class
     */
    public function getClass($index) {
        $name = $this->getMsObj()->getClass($index)->name;    
        
        if(isset($this->children[$name])) {
            return $this->children[$name];
        } else {
            throw new Giscuit_Exception("Class ({$name}) was not found in the haystack.");
        }
    }
    
    /**
     * Returns a class from the layer given a class name.
     * 
     * @param string $name Layer name
     * @return Giscuit_Map_Class
     */
    public function getClassByName($name) {
        if(isset($this->children[$name])) {
            return $this->children[$name];
        } else {
            throw new Giscuit_Exception("Class ({$name}) was not found in the haystack.");
        }
    }
    
    
    /**
     * Remove a class from the map
     * 
     * @param integer $index Index value (0 = first class)
     * @return Giscuit_Map_Class Returns the removed class on success, else null.
     */
    public function removeClass($index) {
        $name = $this->getClass($index)->name;
        if(isset($this->children[$name])) {
            $return = $this->getMsObj()->removeClass($index);
            unset($this->children[$name]);
            return $return;    
        } else {
            throw new Giscuit_Exception("Class ({$name}) was not found in the haystack.");    
        }
    }
    
    /**
     * Delete
     * 
     * @return void
     */
    public function delete()
    {
        for($i = 0; $i < $this->parent->numlayers; $i++) {
            if($this->parent->getLayer($i)->name == $this->name &&
                in_array($this->name, array_keys($this->parent->children))) {
                    
                $this->parent->getMsObj()->removeLayer($i);
                unset($this->parent->children[$this->name]);        
            } else {
                throw new Giscuit_Exception("Layer ({$this->name}) was not found in the haystack.");
            }
        }
    }
    
}